@@ -192,9 +192,9 @@ def BLiP(
 		# Get weights
 		weights = np.array([weight_fn(x) for x in cand_groups])
 
-	# perturb to avoid degeneracy in some cases
+	# perturb to ensure unique solution
+	orig_weights = weights.copy()
 	if perturb:
-		orig_weights = weights.copy()
 		weights = np.array([
 			w*(1 + 0.0001*np.random.uniform()) for w in weights]
 		)
@@ -314,6 +314,7 @@ def BLiP(
 		deterministic=deterministic,
 		problem_status=problem_status,
 		return_problem_status=return_problem_status,
+		verbose=verbose,
 	)
 
 def BLiP_cts(
@@ -411,6 +412,7 @@ def binarize_selections(
 	return_problem_status=False,
 	tol=1e-3,
 	nsample=10,
+	verbose=False,
 ):
 	"""
 	Parameters
@@ -434,9 +436,10 @@ def binarize_selections(
 	output = []
 	if problem_status is None:
 		problem_status = dict(backtracking_iter=0)
+	if error in ['local_fdr', 'pfer', 'fwer']:
+		cand_groups = create_groups._prefilter(cand_groups, max_pep=q)
 
-	# Prune options with zero selection prob
-	# and add options which have selection prob of 1. 
+	# Account for integer solutions
 	nontriv_cand_groups = []
 	for cand_group in cand_groups:
 		if cand_group.data['sprob'] < tol:
@@ -450,9 +453,9 @@ def binarize_selections(
 	ngroups = len(nontriv_cand_groups)
 	problem_status['ngroups_nonint'] = ngroups
 	if ngroups == 0 or ngroups == 1:
-		if ngroups == 1:
-			if not deterministic and np.random.uniform() < nontriv_cand_groups[0].data['sprob']:
-				output.append(nontriv_cand_groups[0])
+		# if ngroups == 1:
+		# 	if not deterministic and np.random.uniform() < nontriv_cand_groups[0].data['sprob']:
+		# 		output.append(nontriv_cand_groups[0])
 		if return_problem_status:
 			return output, problem_status
 		return output
@@ -463,6 +466,10 @@ def binarize_selections(
 	for gj, cand_group in enumerate(nontriv_cand_groups):
 		for feature in cand_group.data['blip-group']:
 			A[gj, feature] = 1
+	if verbose:
+		msg = f"LP had {ngroups} non-integer solutions across {nrel} locations."
+		msg += f" Binarizing using deterministic={deterministic}."
+		print(msg)
 
 	# Sampling method
 	if not deterministic:
@@ -550,17 +557,20 @@ def binarize_selections(
 			A.T @ x <= b
 		]
 		objective = cp.Maximize(((1-peps) * weights) @ x)
+		# PFER / FWER specific constraints
+		# Note: when binary_search=True for FWER, all solutions
+		# are rounded before entering this function.
 		if error in ['pfer', 'fwer']:
-			if error == 'pfer':
-				v_opt = q
-			else:
-				v_opt = sum([cg.pep * cg.data['sprob'] for cg in cand_groups])
+			v_opt = q
 			v_new = v_opt - v_output
 			constraints += [
 				x @ peps <= v_new
 			]
+		# No backtracking required for PFER, FWER, or local FDR
+		if error in ['pfer', 'fwer', 'local_fdr']:
 			problem = cp.Problem(objective=objective, constraints=constraints)
 			problem.solve(solver=BIN_SOLVER)
+		# FDR may require backtracking
 		elif error == 'fdr':
 			# Create output
 			output = sorted(output, key=lambda x: x.pep)
@@ -587,12 +597,14 @@ def binarize_selections(
 						)
 					# Backtrack by getting rid of last group
 					problem_status['backtracking_iter'] += 1
+					if verbose:
+						print(f"Starting backtracking iter={problem_status['backtracking_iter']}")
 					v_output -= output[-1].pep
 					ndisc_out -= 1
 					output = output[0:-1]
 
 		if x.value is None:
-			print(f"stats={problem.status}, nontriv_cand_groups={nontriv_cand_groups}")
+			print(f"status={problem.status}, nontriv_cand_groups={nontriv_cand_groups}")
 
 		for binprob, x in zip(x.value, nontriv_cand_groups):
 			if binprob > 1 - tol:
